###############################################################################
#DExMA: A bioconductor package for performing gene expression meta-analysis 
#with missing genes
###############################################################################

library(DExMA)
data("DExMAExampleData")

#Meta-analysis object creation
phenoGroups = c("condition", "condition", "state", "state")
phenoCases = list(Study1 = "Diseased", Study2 = c("Diseased", "ill"),
    Study3 = "Diseased", Study4 = "ill")
phenoControls = list(Study1 = "Healthy", Study2 = c("Healthy", "control"),
    Study3 = "Healthy", Study4 = "control")
newObjectMA <- createObjectMA(listEX = listMatrixEX, 
    listPheno = listPhenodatas, namePheno = phenoGroups, 
    expGroups = phenoCases, refGroups = phenoControls)
head(newObjectMA[[1]][[1]])
newObjectMA[[1]][[2]]

#Gene annotation
rownames(newObjectMA$Study1$mExpres)[1:20]
newObjectMA <- allSameID(newObjectMA,
    finalID="GeneSymbol", organism = "Homo sapiens")
rownames(newObjectMA$Study1$mExpres)[1:20]

#Quality control
newObjectMA <- dataLog(newObjectMA) #Log-transformation
heterogeneityTest(newObjectMA) #Heterogeneity test

#Missing genes imputation
lapply(maObject, function(x){nrow(x[[1]])}) #Number of genes study 1 before imputation
imputation <- missGenesImput(maObject, k = 7)
maObject_imput <- imputation$objectMA
lapply(maObject_imput, function(x){nrow(x[[1]])}) #Number of genes study 1 after imputation

#Performing gene expression meta-analysis
resultMA <- metaAnalysisDE(objectMA = newObjectMA, typeMethod = "maxP", 
    missAllow = 0.3, proportionData = 0.5)
head(resultMA)
significant <- resultMA[resultMA$FDR <= 0.05,] #Significant genes
rownames(significant)
nrow(significant)

#Visualization of the results
heatObject <- makeHeatmap(objectMA = newObjectMA, resMA = resultMA, typeMethod = "maxP",
    scaling = "zscor", regulation = "all",breaks = c(-2,2),
    fdrSig = 0.05, numSig=50)
head(heatObject)

#EXAMPLE WITH REAL DATA
# 3 Expression matrices (GSE82221Ex, GSE50722Ex , GSE24706Ex)
# 3 Phenodatas (GSE82221Pheno, GSE50722Pheno, GSE24706Pheno)

#Data
load("SLE_Data.RData")

#objectMA creation
#We create two lists, one for the expression matrices and one for the phenodatas:
listMatrices <- list(GSE82221Ex, GSE50772Ex, GSE24706Ex)
listPhenodata <- list(GSE82221Pheno, GSE50772Pheno, GSE24706Pheno)
phenoGroups <- c("Condition","Condition", "Condition")
phenoCases <- list("SLE", "SLE", "SLE")
phenoControls <- list("Healthy", "Healthy", "Healthy")
ObjectMA_SLE <- createObjectMA(listEX = listMatrices, 
    listPheno = listPhenodata, namePheno = phenoGroups, 
    expGroups = phenoCases, refGroups = phenoControls)
names(ObjectMA_SLE) <- c("GSE82221", "GSE50772", "GSE24706")

#Quality control
ObjectMA_SLE <- dataLog(ObjectMA_SLE) #Log2 transformation
heterogeneityTest(ObjectMA_SLE) # check heterogeneity

#Results with only common genes
results_SLE_common <- metaAnalysisDE(objectMA = ObjectMA_SLE,
    typeMethod = "REM", proportionData = 1)
nrow (results_SLE_common)
sig_genes_common <- subset(results_SLE_common, results_SLE_common$FDR<0.05)
nrow(sig_genes_common)

#Results considering genes contained in at least two datasets
results_SLE_prop <- metaAnalysisDE(objectMA = ObjectMA_SLE,
    typeMethod = "REM", proportionData = 0.66)
nrow (results_SLE_prop)
sig_genes_prop <- subset(results_SLE_prop, results_SLE_prop$FDR<0.05)
nrow(sig_genes_prop)

#Results applying missing genes imputation
imputation <- missGenesImput(ObjectMA_SLE, k = 7)
ObjectMA_SLE_imputed <- imputation$objectMA
#Results with genes imputed
results_SLE_imputed <- metaAnalysisDE(objectMA = ObjectMA_SLE_imputed,
    typeMethod = "REM")
nrow(results_SLE_imputed)
sig_genes_imputed <- subset(results_SLE_imputed, results_SLE_imputed$FDR<0.05)
nrow(sig_genes_imputed)

#Visualization of results
#Approach of proportion missing genes
makeHeatmap(objectMA = ObjectMA_SLE, resMA = results_SLE_prop, 
    typeMethod = "REM", scaling = "zscor", 
    regulation = "all", fdrSig = 0.05, numSig = 50)


######################
#APPENDIX 1
######################
setwd("~/Documents/GENYO_JA/Meta_package/DExMA_paper/Archivos_datos")
#Loading gene expression matrix
GSE24706Ex <- as.matrix(read.delim("GSE24706.tsv", header = TRUE, row.names = 1))
GSE50772Ex <- as.matrix(read.delim("GSE50772.tsv", header = TRUE, row.names = 1))
GSE82221Ex <- as.matrix(read.delim("GSE82221_GPL10558.tsv", header = TRUE, row.names = 1))

#Loading pheno data
Pheno <- read.delim("metadata.tsv", header = T, row.names = 1)
GSE24706Pheno <- Pheno[colnames(GSE24706Ex),]
GSE50772Pheno <- Pheno[colnames(GSE50772Ex),]
GSE82221Pheno <- Pheno[colnames(GSE82221Ex),]
